@@ -136,7 +136,7 @@ video_area_realize_cb (GtkWidget *widget,
 	GthMediaViewerPage *self = user_data;
 
 	self->priv->cursor = gdk_cursor_new (GDK_LEFT_PTR);
-	self->priv->cursor_void = gth_cursor_get (gtk_widget_get_window (self->priv->area), GTH_CURSOR_VOID);
+	self->priv->cursor_void = gdk_cursor_new_for_display (gtk_widget_get_display (self->priv->area), GDK_BLANK_CURSOR);
 	if (self->priv->cursor_visible)
 		gdk_window_set_cursor (gtk_widget_get_window (self->priv->area), self->priv->cursor);
 	else
@@ -365,17 +365,14 @@ position_value_changed_cb (GtkAdjustment *adjustment,
 		return;
 
 	current_value = (gint64) (gtk_adjustment_get_value (adjustment) / 100.0 * self->priv->duration);
-	if (! gst_element_seek (self->priv->playbin,
-				self->priv->rate,
-			        GST_FORMAT_TIME,
-			        GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT,
-			        GST_SEEK_TYPE_SET,
-			        current_value,
-			        GST_SEEK_TYPE_NONE,
-			        0.0))
-	{
-		g_warning ("seek failed");
-	}
+	gst_element_seek (self->priv->playbin,
+			  self->priv->rate,
+			  GST_FORMAT_TIME,
+			  GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT,
+			  GST_SEEK_TYPE_SET,
+			  current_value,
+			  GST_SEEK_TYPE_NONE,
+			  0.0);
 
 	s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (current_value));
 	gtk_label_set_text (GTK_LABEL (GET_WIDGET ("label_position")), s);
@@ -429,10 +426,7 @@ update_playback_info (GthMediaViewerPage *self)
 {
 	char *playback_info;
 
-	if (self->priv->playing)
-		playback_info = g_strdup_printf ("@%2.2f", self->priv->rate);
-	else
-		playback_info = g_strdup ("");
+	playback_info = g_strdup_printf ("@%2.2f", self->priv->rate);
 	g_file_info_set_attribute_string (gth_browser_get_current_file (self->priv->browser)->info, "gthumb::statusbar-extra-info", playback_info);
 	gth_browser_update_statusbar_file_info (self->priv->browser);
 
@@ -483,7 +477,27 @@ button_play_clicked_cb (GtkButton *button,
 	if (! self->priv->playing) {
 		if (! self->priv->paused) {
 			gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
-			gst_element_seek_simple (self->priv->playbin, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, 0);
+			gst_element_seek (self->priv->playbin,
+					  self->priv->rate,
+					  GST_FORMAT_TIME,
+					  GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE,
+					  GST_SEEK_TYPE_SET,
+					  0.0,
+					  GST_SEEK_TYPE_NONE,
+					  0.0);
+		}
+		else {
+			gint64 current_value;
+
+			current_value = (gint64) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_position"))) / 100.0 * self->priv->duration);
+			gst_element_seek (self->priv->playbin,
+					  self->priv->rate,
+					  GST_FORMAT_TIME,
+					  GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE,
+					  GST_SEEK_TYPE_SET,
+					  current_value,
+					  GST_SEEK_TYPE_NONE,
+					  0.0);
 		}
 		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
 	}
@@ -1181,8 +1195,9 @@ gth_media_viewer_page_real_update_sensitivity (GthViewerPage *base)
 {
 	GthMediaViewerPage *self = (GthMediaViewerPage *) base;
 
-	gtk_widget_set_sensitive (GET_WIDGET ("button_play_slower"), self->priv->playing);
-	gtk_widget_set_sensitive (GET_WIDGET ("button_play_faster"), self->priv->playing);
+	/*gtk_widget_set_sensitive (GET_WIDGET ("button_play_slower"), self->priv->playing);
+	gtk_widget_set_sensitive (GET_WIDGET ("button_play_faster"), self->priv->playing);*/
+
 	gtk_widget_set_sensitive (GET_WIDGET ("volume_box"), self->priv->has_audio);
 	set_action_sensitive (self, "MediaViewer_Screenshot", self->priv->has_video);
 }
@@ -1221,6 +1236,19 @@ gth_media_viewer_page_real_revert (GthViewerPage *base)
 }
 
 
+static void
+gth_media_viewer_page_real_update_info (GthViewerPage *base,
+				        GthFileData   *file_data)
+{
+	GthMediaViewerPage *self = GTH_MEDIA_VIEWER_PAGE (base);
+
+	if (! _g_file_equal (self->priv->file_data->file, file_data->file))
+		return;
+	_g_object_unref (self->priv->file_data);
+	self->priv->file_data = gth_file_data_dup (file_data);
+}
+
+
 static void
 gth_media_viewer_page_finalize (GObject *obj)
 {
@@ -1272,6 +1300,7 @@ gth_viewer_page_interface_init (GthViewerPageIface *iface)
 	iface->save = gth_media_viewer_page_real_save;
 	iface->save_as = gth_media_viewer_page_real_save_as;
 	iface->revert = gth_media_viewer_page_real_revert;
+	iface->update_info = gth_media_viewer_page_real_update_info;
 }
 
 
