@@ -30,21 +30,9 @@
 #include "preferences.h"
 
 
-#define DEFAULT_FONT_NAME "sans normal 10"
 #define GET_WIDGET(name) _gtk_builder_get_widget (self->priv->builder, (name))
 
 
-enum {
-	IMAGES_PER_PAGE_1,
-	IMAGES_PER_PAGE_2,
-	IMAGES_PER_PAGE_4,
-	IMAGES_PER_PAGE_8,
-	IMAGES_PER_PAGE_16,
-	IMAGES_PER_PAGE_32,
-	N_IMAGES_PER_PAGE
-};
-static int n_rows_for_ipp[N_IMAGES_PER_PAGE] = { 1, 2, 2, 4, 4, 8 };
-static int n_cols_for_ipp[N_IMAGES_PER_PAGE] = { 1, 1, 2, 2, 4, 4 };
 static gpointer parent_class = NULL;
 
 
@@ -55,6 +43,7 @@ struct _GthImagePrintJobPrivate {
 	GtkBuilder         *builder;
 	GtkWidget          *caption_chooser;
 	GthImageInfo       *selected;
+	char               *event_name;
 
 	gulong              rotation_combobox_changed_event;
 	gulong              scale_adjustment_value_changed_event;
@@ -69,25 +58,34 @@ struct _GthImagePrintJobPrivate {
 
 	GthImageInfo      **images;
 	int                 n_images;
-	int                 requested_images_per_page;
+	int                 n_rows;
+	int                 n_columns;
 	int                 image_width;
 	int                 image_height;
 	GtkPageSetup       *page_setup;
 	char               *caption_attributes;
-	char               *font_name;
+	char               *caption_font_name;
+	char               *header_font_name;
+	char               *footer_font_name;
 	double              scale_factor;
 	int                 dpi;
+	char               *header_template;
+	char               *footer_template;
+	char               *header;
+	char               *footer;
 
 	/* layout info */
 
 	GthTask            *task;
-	int                 real_images_per_page;
 	double              max_image_width;
 	double		    max_image_height;
 	double              x_padding;
 	double              y_padding;
+	GthRectangle        header_rectangle;
+	GthRectangle        footer_rectangle;
 	int                 n_pages;
 	int                 current_page;
+	gboolean            printing;
 };
 
 
@@ -100,14 +98,21 @@ gth_image_print_job_finalize (GObject *base)
 	self = GTH_IMAGE_PRINT_JOB (base);
 
 	_g_object_unref (self->priv->task);
-	_g_object_unref (self->priv->print_operation);
-	_g_object_unref (self->priv->builder);
+	g_free (self->priv->footer);
+	g_free (self->priv->header);
+	g_free (self->priv->footer_template);
+	g_free (self->priv->header_template);
+	g_free (self->priv->footer_font_name);
+	g_free (self->priv->header_font_name);
+	g_free (self->priv->caption_font_name);
+	g_free (self->priv->caption_attributes);
+	_g_object_unref (self->priv->page_setup);
 	for (i = 0; i < self->priv->n_images; i++)
 		gth_image_info_unref (self->priv->images[i]);
 	g_free (self->priv->images);
-	_g_object_unref (self->priv->page_setup);
-	g_free (self->priv->caption_attributes);
-	g_free (self->priv->font_name);
+	_g_object_unref (self->priv->print_operation);
+	_g_object_unref (self->priv->builder);
+	g_free (self->priv->event_name);
 
 	G_OBJECT_CLASS (parent_class)->finalize (base);
 }
@@ -130,15 +135,26 @@ static void
 gth_image_print_job_init (GthImagePrintJob *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMAGE_PRINT_JOB, GthImagePrintJobPrivate);
+	self->priv->event_name = NULL;
 	self->priv->builder = NULL;
 	self->priv->task = NULL;
 	self->priv->page_setup = NULL;
 	self->priv->current_page = 0;
 	self->priv->caption_attributes = eel_gconf_get_string (PREF_IMAGE_PRINT_CAPTION, "");
-	self->priv->font_name = eel_gconf_get_string (PREF_IMAGE_PRINT_FONT_NAME, DEFAULT_FONT_NAME);
+	self->priv->caption_font_name = eel_gconf_get_string (PREF_IMAGE_PRINT_FONT_NAME, DEFAULT_CAPTION_FONT_NAME);
+	self->priv->header_font_name = eel_gconf_get_string (PREF_IMAGE_PRINT_HEADER_FONT_NAME, DEFAULT_HEADER_FONT_NAME);
+	self->priv->footer_font_name = eel_gconf_get_string (PREF_IMAGE_PRINT_FOOTER_FONT_NAME, DEFAULT_FOOTER_FONT_NAME);
 	self->priv->selected = NULL;
-	self->priv->requested_images_per_page = eel_gconf_get_integer (PREF_IMAGE_PRINT_IMAGES_PER_PAGE, 1);
+	self->priv->n_rows = eel_gconf_get_integer (PREF_IMAGE_PRINT_N_ROWS, 1);
+	self->priv->n_columns = eel_gconf_get_integer (PREF_IMAGE_PRINT_N_COLUMNS, 1);
 	self->priv->unit = eel_gconf_get_enum (PREF_IMAGE_PRINT_UNIT, GTH_TYPE_METRIC, GTH_METRIC_PIXELS);
+	self->priv->header_rectangle.height = 0;
+	self->priv->footer_rectangle.height = 0;
+	self->priv->header_template = eel_gconf_get_string (PREF_IMAGE_PRINT_HEADER, DEFAULT_HEADER);
+	self->priv->footer_template = eel_gconf_get_string (PREF_IMAGE_PRINT_FOOTER, DEFAULT_FOOTER);
+	self->priv->header = NULL;
+	self->priv->footer = NULL;
+	self->priv->printing = FALSE;
 }
 
 
@@ -171,23 +187,147 @@ gth_image_print_job_get_type (void)
 
 
 static double
-_log2 (double x)
+get_text_height (GthImagePrintJob *self,
+		 PangoLayout      *pango_layout,
+		 const char       *text,
+		 int               width)
 {
-	return log (x) / log (2);
+	PangoRectangle logical_rect;
+
+	if (text == NULL)
+		return 0.0;
+
+	pango_layout_set_text (pango_layout, text, -1);
+	pango_layout_set_width (pango_layout, width * self->priv->scale_factor * PANGO_SCALE);
+	pango_layout_get_pixel_extents (pango_layout, NULL, &logical_rect);
+
+	return logical_rect.height / self->priv->scale_factor;
 }
 
 
-static int
-get_combo_box_index_from_ipp (int value)
+static void
+gth_image_print_job_set_font_options (GthImagePrintJob *self,
+				      PangoLayout      *pango_layout,
+				      const char       *font_name,
+				      gboolean          preview)
 {
-	return (int) floor (_log2 (value) + 0.5);
+	PangoFontDescription *font_desc;
+	double                size_in_points;
+	cairo_font_options_t *options;
+	PangoContext         *pango_context;
+
+	pango_layout_set_wrap (pango_layout, PANGO_WRAP_WORD_CHAR);
+	pango_layout_set_justify (pango_layout, FALSE);
+	pango_layout_set_alignment (pango_layout, PANGO_ALIGN_CENTER);
+
+	font_desc = pango_font_description_from_string (font_name);
+	if (preview)
+		self->priv->scale_factor = 2.83;
+	else
+		self->priv->scale_factor = 1.0;
+
+	size_in_points = (double) pango_font_description_get_size (font_desc) / PANGO_SCALE;
+	pango_font_description_set_absolute_size (font_desc, size_in_points * PANGO_SCALE);
+	pango_layout_set_font_description (pango_layout, font_desc);
+
+	options = cairo_font_options_create ();
+	cairo_font_options_set_hint_metrics (options, CAIRO_HINT_METRICS_OFF);
+	pango_context = pango_layout_get_context (pango_layout);
+	pango_cairo_context_set_font_options (pango_context, options);
+
+	cairo_font_options_destroy (options);
+	pango_font_description_free (font_desc);
 }
 
 
-static int
-get_ipp_from_combo_box_index (int idx)
+static gboolean
+template_eval_cb (const GMatchInfo *match_info,
+		  GString          *result,
+		  gpointer          user_data)
 {
-	return (int) pow (2, idx);
+	GthImagePrintJob *self = user_data;
+	char             *r = NULL;
+	char             *match;
+
+	match = g_match_info_fetch (match_info, 0);
+	if (strcmp (match, "%p") == 0) {
+		r = g_strdup_printf ("%d", self->priv->current_page + 1);
+	}
+	else if (strcmp (match, "%P") == 0) {
+		r = g_strdup_printf ("%d", self->priv->n_pages);
+	}
+	else if (strcmp (match, "%F") == 0) {
+		r = g_strdup_printf ("%d", self->priv->n_images);
+	}
+	else if (strncmp (match, "%D", 2) == 0) {
+		GTimeVal   timeval;
+		GRegex    *re;
+		char     **a;
+		char      *format = NULL;
+
+		g_get_current_time (&timeval);
+
+		/* Get the date format */
+
+		re = g_regex_new ("%[A-Z]\\{([^}]+)\\}", 0, 0, NULL);
+		a = g_regex_split (re, match, 0);
+		if (g_strv_length (a) >= 2)
+			format = g_strstrip (a[1]);
+		r = _g_time_val_strftime (&timeval, format);
+
+		g_strfreev (a);
+		g_regex_unref (re);
+	}
+	else if (strcmp (match, "%E") == 0) {
+		if (self->priv->event_name != NULL)
+			r = g_strdup (self->priv->event_name);
+		else
+			r = g_strdup ("");
+	}
+
+	if (r != NULL)
+		g_string_append (result, r);
+
+	g_free (r);
+	g_free (match);
+
+	return FALSE;
+}
+
+
+static char *
+get_text_from_template (GthImagePrintJob *self,
+			const char       *text)
+{
+	GRegex *re;
+	char   *new_text;
+
+	if (text == NULL)
+		return NULL;
+
+	if (g_utf8_strchr (text,  -1, '%') == NULL)
+		return g_strdup (text);
+
+	re = g_regex_new ("%[DEFPp](\\{[^}]+\\})?", 0, 0, NULL);
+	new_text = g_regex_replace_eval (re, text, -1, 0, 0, template_eval_cb, self, NULL);
+	g_regex_unref (re);
+
+	return new_text;
+}
+
+
+static void
+update_header_and_footer_texts (GthImagePrintJob *self)
+{
+	g_free (self->priv->header);
+	self->priv->header = NULL;
+	if ((self->priv->header_template != NULL) && (g_strcmp0 (self->priv->header_template, "") != 0))
+		self->priv->header = get_text_from_template (self, self->priv->header_template);
+
+	g_free (self->priv->footer);
+	self->priv->footer = NULL;
+	if ((self->priv->footer_template != NULL) && (g_strcmp0 (self->priv->footer_template, "") != 0))
+		self->priv->footer = get_text_from_template (self, self->priv->footer_template);
 }
 
 
@@ -195,57 +335,94 @@ static void
 gth_image_print_job_update_layout_info (GthImagePrintJob   *self,
 				        gdouble             page_width,
 				        gdouble             page_height,
-				        GtkPageOrientation  orientation)
+				        GtkPageOrientation  orientation,
+				        PangoLayout        *pango_layout,
+				        gboolean            preview)
 {
-	int idx;
-	int rows;
-	int cols;
-	int current_page;
-	int current_row;
-	int current_col;
-	int i;
+	gboolean height_changed = FALSE;
+	int      height;
+	int      rows;
+	int      columns;
+	int      current_page;
+	int      current_row;
+	int      current_column;
+	int      i;
 
-	self->priv->real_images_per_page = self->priv->requested_images_per_page;
 	self->priv->x_padding = page_width / 40.0;
 	self->priv->y_padding = page_height / 40.0;
 
-	idx = get_combo_box_index_from_ipp (self->priv->real_images_per_page);
-	rows = n_rows_for_ipp[idx];
-	cols = n_cols_for_ipp[idx];
+	/* header */
+
+	gth_image_print_job_set_font_options (self, pango_layout, self->priv->header_font_name, preview);
+	height = get_text_height (self, pango_layout, self->priv->header, page_width);
+	if (height != self->priv->header_rectangle.height)
+		height_changed = TRUE;
+	self->priv->header_rectangle.height = height;
+	self->priv->header_rectangle.y = 0.0;
+	self->priv->header_rectangle.x = 0.0;
+	self->priv->header_rectangle.width = page_width;
+
+	/* footer */
+
+	gth_image_print_job_set_font_options (self, pango_layout, self->priv->footer_font_name, preview);
+	height = get_text_height (self, pango_layout, self->priv->footer, page_width);
+	if (height != self->priv->footer_rectangle.height)
+		height_changed = TRUE;
+	self->priv->footer_rectangle.height = height;
+	self->priv->footer_rectangle.y = page_height - self->priv->footer_rectangle.height;
+	self->priv->footer_rectangle.x = 0.0;
+	self->priv->footer_rectangle.width = page_width;
+
+	/* images */
+
+	if (! self->priv->printing && height_changed) {
+		for (i = 0; i < self->priv->n_images; i++)
+			gth_image_info_reset (self->priv->images[i]);
+	}
+
+	rows = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("rows_spinbutton")));
+	columns = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("columns_spinbutton")));
 	if ((orientation == GTK_PAGE_ORIENTATION_LANDSCAPE)
 	    || (orientation == GTK_PAGE_ORIENTATION_REVERSE_LANDSCAPE))
 	{
 		int tmp = rows;
-		rows = cols;
-		cols = tmp;
+		rows = columns;
+		columns = tmp;
 	}
 
-	self->priv->max_image_width = (page_width - ((cols - 1) * self->priv->x_padding)) / cols;
+	if (self->priv->header_rectangle.height > 0)
+		page_height -= self->priv->header_rectangle.height + self->priv->y_padding;
+	if (self->priv->footer_rectangle.height > 0)
+		page_height -= self->priv->footer_rectangle.height + self->priv->y_padding;
+
+	self->priv->n_rows = rows;
+	self->priv->n_columns = columns;
+	self->priv->max_image_width = (page_width - ((columns - 1) * self->priv->x_padding)) / columns;
 	self->priv->max_image_height = (page_height - ((rows - 1) * self->priv->y_padding)) / rows;
 
-	self->priv->n_pages = MAX ((int) ceil ((double) self->priv->n_images / self->priv->real_images_per_page), 1);
+	self->priv->n_pages = MAX ((int) ceil ((double) self->priv->n_images / (self->priv->n_rows * self->priv->n_columns)), 1);
 	if (self->priv->current_page >= self->priv->n_pages)
 		self->priv->current_page = self->priv->n_pages - 1;
 
 	current_page = 0;
 	current_row = 1;
-	current_col = 1;
+	current_column = 1;
 	for (i = 0; i < self->priv->n_images; i++) {
 		GthImageInfo *image_info = self->priv->images[i];
 
 		image_info->page = current_page;
-		image_info->col = current_col;
+		image_info->col = current_column;
 		image_info->row = current_row;
 
-		current_col++;
-		if (current_col > cols) {
+		current_column++;
+		if (current_column > columns) {
 			current_row++;
-			current_col = 1;
+			current_column = 1;
 		}
 
 		if (current_row > rows) {
 			current_page++;
-			current_col = 1;
+			current_column = 1;
 			current_row = 1;
 		}
 	}
@@ -271,6 +448,8 @@ gth_image_print_job_update_image_layout (GthImagePrintJob    *self,
 
 	image_info->boundary.x = (image_info->col - 1) * (self->priv->max_image_width + self->priv->x_padding);
 	image_info->boundary.y = (image_info->row - 1) * (self->priv->max_image_height + self->priv->y_padding);
+	if (self->priv->header_rectangle.height > 0)
+		image_info->boundary.y += self->priv->header_rectangle.height + self->priv->y_padding;
 	image_info->boundary.width = self->priv->max_image_width;
 	image_info->boundary.height = self->priv->max_image_height;
 
@@ -368,66 +547,20 @@ gth_image_print_job_update_image_layout (GthImagePrintJob    *self,
 }
 
 
-static void
-gth_image_print_job_set_font_options (GthImagePrintJob *self,
-				      PangoLayout      *pango_layout,
-				      gboolean          preview)
-{
-	PangoFontDescription *font_desc;
-	double                size_in_points;
-	cairo_font_options_t *options;
-	PangoContext         *pango_context;
-
-	pango_layout_set_wrap (pango_layout, PANGO_WRAP_WORD_CHAR);
-	pango_layout_set_justify (pango_layout, FALSE);
-	pango_layout_set_alignment (pango_layout, PANGO_ALIGN_CENTER);
-
-	font_desc = pango_font_description_from_string (self->priv->font_name);
-	if (preview)
-		self->priv->scale_factor = 2.83;
-	else
-		self->priv->scale_factor = 1.0;
-
-	size_in_points = (double) pango_font_description_get_size (font_desc) / PANGO_SCALE;
-	pango_font_description_set_absolute_size (font_desc, size_in_points * PANGO_SCALE);
-	pango_layout_set_font_description (pango_layout, font_desc);
-
-	options = cairo_font_options_create ();
-	cairo_font_options_set_hint_metrics (options, CAIRO_HINT_METRICS_OFF);
-	pango_context = pango_layout_get_context (pango_layout);
-	pango_cairo_context_set_font_options (pango_context, options);
-
-	cairo_font_options_destroy (options);
-	pango_font_description_free (font_desc);
-}
-
-
-static PangoLayout *
-gth_image_print_job_create_pango_layout (GthImagePrintJob *self,
-					 gboolean          preview)
-{
-	PangoLayout *pango_layout;
-
-	pango_layout = gtk_widget_create_pango_layout (GTK_WIDGET (self->priv->browser), NULL);
-	gth_image_print_job_set_font_options (self, pango_layout, preview);
-
-	return pango_layout;
-}
-
-
 static void
 gth_image_print_job_update_page_layout (GthImagePrintJob   *self,
 					int                 page,
 					gdouble             page_width,
 					gdouble             page_height,
 					GtkPageOrientation  orientation,
+					PangoLayout        *pango_layout,
 					gboolean            preview)
 {
-	PangoLayout  *pango_layout;
-	char        **attributes_v;
-	int           i;
+	char **attributes_v;
+	int    i;
+
+	gth_image_print_job_set_font_options (self, pango_layout, self->priv->caption_font_name, preview);
 
-	pango_layout = gth_image_print_job_create_pango_layout (self, preview);
 	attributes_v = g_strsplit (self->priv->caption_attributes, ",", -1);
 	for (i = 0; i < self->priv->n_images; i++) {
 		GthImageInfo *image_info = self->priv->images[i];
@@ -446,7 +579,6 @@ gth_image_print_job_update_page_layout (GthImagePrintJob   *self,
 	}
 
 	g_strfreev (attributes_v);
-	g_object_unref (pango_layout);
 }
 
 
@@ -456,8 +588,26 @@ gth_image_print_job_update_layout (GthImagePrintJob   *self,
 			  	   gdouble             page_height,
 			  	   GtkPageOrientation  orientation)
 {
-	gth_image_print_job_update_layout_info (self, page_width, page_height, orientation);
-	gth_image_print_job_update_page_layout (self, self->priv->current_page, page_width, page_height, orientation, TRUE);
+	PangoLayout *pango_layout;
+
+	update_header_and_footer_texts (self);
+
+	pango_layout = gtk_widget_create_pango_layout (GTK_WIDGET (self->priv->browser), NULL);
+	gth_image_print_job_update_layout_info (self,
+						page_width,
+						page_height,
+						orientation,
+						pango_layout,
+						TRUE);
+	gth_image_print_job_update_page_layout (self,
+						self->priv->current_page,
+						page_width,
+						page_height,
+						orientation,
+						pango_layout,
+						TRUE);
+
+	g_object_unref (pango_layout);
 }
 
 
@@ -572,7 +722,46 @@ gth_image_print_job_paint (GthImagePrintJob *self,
 {
 	int i;
 
-	gth_image_print_job_set_font_options (self, pango_layout, preview);
+	if (self->priv->header != NULL) {
+		gth_image_print_job_set_font_options (self, pango_layout, self->priv->header_font_name, preview);
+
+		cairo_save (cr);
+
+		pango_layout_set_width (pango_layout, self->priv->header_rectangle.width * self->priv->scale_factor * PANGO_SCALE);
+		pango_layout_set_text (pango_layout, self->priv->header, -1);
+
+		cairo_move_to (cr, x_offset + self->priv->header_rectangle.x, y_offset + self->priv->header_rectangle.y);
+		if (preview)
+			cairo_scale (cr, 1.0 / self->priv->scale_factor, 1.0 / self->priv->scale_factor);
+
+		pango_cairo_layout_path (cr, pango_layout);
+		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+		cairo_fill (cr);
+
+		cairo_restore (cr);
+	}
+
+	if (self->priv->footer != NULL) {
+		gth_image_print_job_set_font_options (self, pango_layout, self->priv->footer_font_name, preview);
+
+		cairo_save (cr);
+
+		pango_layout_set_width (pango_layout, self->priv->footer_rectangle.width * self->priv->scale_factor * PANGO_SCALE);
+		pango_layout_set_text (pango_layout, self->priv->footer, -1);
+
+		cairo_move_to (cr, x_offset + self->priv->footer_rectangle.x, y_offset + self->priv->footer_rectangle.y);
+		if (preview)
+			cairo_scale (cr, 1.0 / self->priv->scale_factor, 1.0 / self->priv->scale_factor);
+
+		pango_cairo_layout_path (cr, pango_layout);
+		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+		cairo_fill (cr);
+
+		cairo_restore (cr);
+	}
+
+	gth_image_print_job_set_font_options (self, pango_layout, self->priv->caption_font_name, preview);
+
 	for (i = 0; i < self->priv->n_images; i++) {
 		GthImageInfo *image_info = self->priv->images[i];
 		GdkPixbuf    *fullsize_pixbuf;
@@ -827,7 +1016,8 @@ gth_image_print_job_update_image_preview (GthImagePrintJob *self,
 	PangoLayout  *pango_layout;
 	char        **attributes_v;
 
-	pango_layout = gth_image_print_job_create_pango_layout (self, TRUE);
+	pango_layout = gtk_widget_create_pango_layout (GTK_WIDGET (self->priv->browser), NULL);
+
 	attributes_v = g_strsplit (self->priv->caption_attributes, ",", -1);
 	gth_image_print_job_update_image_layout (self,
 						 image_info,
@@ -988,13 +1178,27 @@ preview_button_press_event_cb (GtkWidget      *widget,
 
 
 static void
-ipp_combobox_changed_cb (GtkComboBox *widget,
-			 gpointer     user_data)
+rows_spinbutton_changed_cb (GtkSpinButton *widget,
+			    gpointer       user_data)
 {
 	GthImagePrintJob *self = user_data;
 	int               i;
 
-	self->priv->requested_images_per_page = get_ipp_from_combo_box_index (gtk_combo_box_get_active (widget));
+	self->priv->n_rows = gtk_spin_button_get_value_as_int (widget);
+	for (i = 0; i < self->priv->n_images; i++)
+		gth_image_info_reset (self->priv->images[i]);
+	gth_image_print_job_update_preview (self);
+}
+
+
+static void
+columns_spinbutton_changed_cb (GtkSpinButton *widget,
+			       gpointer       user_data)
+{
+	GthImagePrintJob *self = user_data;
+	int               i;
+
+	self->priv->n_columns = gtk_spin_button_get_value_as_int (widget);
 	for (i = 0; i < self->priv->n_images; i++)
 		gth_image_info_reset (self->priv->images[i]);
 	gth_image_print_job_update_preview (self);
@@ -1026,12 +1230,14 @@ prev_page_button_clicked_cb (GtkWidget *widget,
 
 
 static void
-metadata_ready_cb (GList    *files,
-		   GError   *error,
-		   gpointer  user_data)
+metadata_ready_cb (GObject      *source_object,
+                   GAsyncResult *result,
+                   gpointer      user_data)
 {
 	GthImagePrintJob *self = user_data;
+	GError           *error;
 
+	_g_query_metadata_finish (result, &error);
 	gth_image_print_job_update_preview (self);
 }
 
@@ -1109,19 +1315,54 @@ unit_combobox_changed_cb (GtkComboBox *combo_box,
 
 
 static void
-caption_fontbutton_font_set_cb (GtkFontButton *font_button,
-				gpointer       user_data)
+header_entry_changed_cb (GtkEditable *editable,
+			 gpointer     user_data)
 {
 	GthImagePrintJob *self = user_data;
 
-	g_free (self->priv->font_name);
-	self->priv->font_name = g_strdup (gtk_font_button_get_font_name (font_button));
-	eel_gconf_set_string (PREF_IMAGE_PRINT_FONT_NAME, self->priv->font_name);
+	_g_strset (&self->priv->header_template, gtk_entry_get_text (GTK_ENTRY (editable)));
+	if (g_strcmp0 (self->priv->header_template, "") == 0) {
+		g_free (self->priv->header_template);
+		self->priv->header_template = NULL;
+	}
 
 	gth_image_print_job_update_preview (self);
 }
 
 
+static void
+footer_entry_changed_cb (GtkEditable *editable,
+			 gpointer     user_data)
+{
+	GthImagePrintJob *self = user_data;
+
+	_g_strset (&self->priv->footer_template, gtk_entry_get_text (GTK_ENTRY (editable)));
+	if (g_strcmp0 (self->priv->footer_template, "") == 0) {
+		g_free (self->priv->footer_template);
+		self->priv->footer_template = NULL;
+	}
+
+	gth_image_print_job_update_preview (self);
+}
+
+
+static void
+header_or_footer_icon_press_cb (GtkEntry            *entry,
+				GtkEntryIconPosition icon_pos,
+				GdkEvent            *event,
+				gpointer             user_data)
+{
+	GthImagePrintJob *self = user_data;
+	GtkWidget        *help_table;
+
+	help_table = GET_WIDGET ("page_footer_help_table");
+	if (gtk_widget_get_visible (help_table))
+		gtk_widget_hide (help_table);
+	else
+		gtk_widget_show (help_table);
+}
+
+
 static void
 rotation_combobox_changed_cb (GtkComboBox *combo_box,
 			      gpointer     user_data)
@@ -1275,9 +1516,10 @@ operation_create_custom_widget_cb (GtkPrintOperation *operation,
 
 	gth_metadata_chooser_set_selection (GTH_METADATA_CHOOSER (self->priv->caption_chooser), self->priv->caption_attributes);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")), self->priv->unit);
-	gtk_font_button_set_font_name (GTK_FONT_BUTTON (GET_WIDGET ("caption_fontbutton")), self->priv->font_name);
 
-	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("ipp_combobox")), get_combo_box_index_from_ipp (self->priv->requested_images_per_page));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("rows_spinbutton")), self->priv->n_rows);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("columns_spinbutton")), self->priv->n_columns);
+
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")), eel_gconf_get_enum (PREF_IMAGE_PRINT_UNIT, GTH_TYPE_METRIC, GTH_METRIC_PIXELS));
 
 	g_signal_connect (GET_WIDGET ("preview_drawingarea"),
@@ -1296,9 +1538,13 @@ operation_create_custom_widget_cb (GtkPrintOperation *operation,
 			  "button-press-event",
 			  G_CALLBACK (preview_button_press_event_cb),
 			  self);
-	g_signal_connect (GET_WIDGET ("ipp_combobox"),
-			  "changed",
-	                  G_CALLBACK (ipp_combobox_changed_cb),
+	g_signal_connect (GET_WIDGET ("rows_spinbutton"),
+			  "value-changed",
+	                  G_CALLBACK (rows_spinbutton_changed_cb),
+	                  self);
+	g_signal_connect (GET_WIDGET ("columns_spinbutton"),
+			  "value-changed",
+	                  G_CALLBACK (columns_spinbutton_changed_cb),
 	                  self);
 	g_signal_connect (GET_WIDGET ("next_page_button"),
 			  "clicked",
@@ -1316,10 +1562,22 @@ operation_create_custom_widget_cb (GtkPrintOperation *operation,
 			  "changed",
 	                  G_CALLBACK (unit_combobox_changed_cb),
 	                  self);
-	g_signal_connect (GET_WIDGET ("caption_fontbutton"),
-			  "font-set",
-			  G_CALLBACK (caption_fontbutton_font_set_cb),
-			  self);
+	g_signal_connect (GET_WIDGET ("header_entry"),
+			  "changed",
+	                  G_CALLBACK (header_entry_changed_cb),
+	                  self);
+	g_signal_connect (GET_WIDGET ("footer_entry"),
+			  "changed",
+	                  G_CALLBACK (footer_entry_changed_cb),
+	                  self);
+	g_signal_connect (GET_WIDGET ("header_entry"),
+			  "icon-press",
+	                  G_CALLBACK (header_or_footer_icon_press_cb),
+	                  self);
+	g_signal_connect (GET_WIDGET ("footer_entry"),
+			  "icon-press",
+	                  G_CALLBACK (header_or_footer_icon_press_cb),
+	                  self);
 
 	self->priv->rotation_combobox_changed_event =
 			g_signal_connect (GET_WIDGET ("rotation_combobox"),
@@ -1397,6 +1655,9 @@ operation_update_custom_widget_cb (GtkPrintOperation *operation,
 	gtk_widget_set_size_request (GET_WIDGET ("preview_drawingarea"),
 				     gtk_page_setup_get_paper_width (setup, GTK_UNIT_MM),
 				     gtk_page_setup_get_paper_height (setup, GTK_UNIT_MM));
+	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("header_entry")), self->priv->header_template);
+	gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("footer_entry")), self->priv->footer_template);
+
 	for (i = 0; i < self->priv->n_images; i++)
 		gth_image_info_reset (self->priv->images[i]);
 	gth_image_print_job_update_preview (self);
@@ -1410,8 +1671,11 @@ operation_custom_widget_apply_cb (GtkPrintOperation *operation,
 {
 	GthImagePrintJob *self = user_data;
 
-	eel_gconf_set_integer (PREF_IMAGE_PRINT_IMAGES_PER_PAGE, self->priv->requested_images_per_page);
+	eel_gconf_set_integer (PREF_IMAGE_PRINT_N_ROWS, self->priv->n_rows);
+	eel_gconf_set_integer (PREF_IMAGE_PRINT_N_COLUMNS, self->priv->n_columns);
 	eel_gconf_set_enum (PREF_IMAGE_PRINT_UNIT, GTH_TYPE_METRIC, gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox"))));
+	eel_gconf_set_string (PREF_IMAGE_PRINT_HEADER, gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("header_entry"))));
+	eel_gconf_set_string (PREF_IMAGE_PRINT_FOOTER, gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("footer_entry"))));
 }
 
 
@@ -1423,6 +1687,7 @@ print_operation_begin_print_cb (GtkPrintOperation *operation,
 	GthImagePrintJob *self = user_data;
 	GtkPrintSettings *settings;
 	char             *filename;
+	PangoLayout      *pango_layout;
 
 	_g_object_unref (self->priv->page_setup);
 	self->priv->page_setup = gtk_page_setup_copy (gtk_print_context_get_page_setup (context));
@@ -1436,11 +1701,18 @@ print_operation_begin_print_cb (GtkPrintOperation *operation,
 	gtk_page_setup_to_file (self->priv->page_setup, filename, NULL);
 	g_free (filename);
 
+	self->priv->printing = TRUE;
+
+	pango_layout = gtk_print_context_create_pango_layout (context);
 	gth_image_print_job_update_layout_info (self,
 						gtk_print_context_get_width (context),
 						gtk_print_context_get_height (context),
-						gtk_page_setup_get_orientation (self->priv->page_setup));
+						gtk_page_setup_get_orientation (self->priv->page_setup),
+						pango_layout,
+						FALSE);
 	gtk_print_operation_set_n_pages (operation, self->priv->n_pages);
+
+	g_object_unref (pango_layout);
 }
 
 
@@ -1451,20 +1723,24 @@ print_operation_draw_page_cb (GtkPrintOperation *operation,
 			      gpointer           user_data)
 {
 	GthImagePrintJob *self = user_data;
-	GtkPageSetup     *setup;
 	cairo_t          *cr;
 	PangoLayout      *pango_layout;
+	GtkPageSetup     *setup;
 
+	cr = gtk_print_context_get_cairo_context (context);
+	pango_layout = gtk_print_context_create_pango_layout (context);
 	setup = gtk_print_context_get_page_setup (context);
+
+	self->priv->current_page = page_nr;
+	update_header_and_footer_texts (self);
+
 	gth_image_print_job_update_page_layout (self,
 						page_nr,
 						gtk_print_context_get_width (context),
 						gtk_print_context_get_height (context),
 						gtk_page_setup_get_orientation (setup),
+						pango_layout,
 						FALSE);
-
-	cr = gtk_print_context_get_cairo_context (context);
-	pango_layout = gtk_print_context_create_pango_layout (context);
 	gth_image_print_job_paint (self,
 				   cr,
 				   pango_layout,
@@ -1509,6 +1785,7 @@ GthImagePrintJob *
 gth_image_print_job_new (GList        *file_data_list,
 			 GthFileData  *current,
 			 GdkPixbuf    *current_image,
+			 const char   *event_name,
 			 GError      **error)
 {
 	GthImagePrintJob *self;
@@ -1534,6 +1811,8 @@ gth_image_print_job_new (GList        *file_data_list,
 	}
 	self->priv->images[n] = NULL;
 	self->priv->n_images = n;
+	self->priv->event_name = g_strdup (event_name);
+
 	self->priv->image_width = 0;
 	self->priv->image_height = 0;
 
@@ -1546,7 +1825,7 @@ gth_image_print_job_new (GList        *file_data_list,
 
 	self->priv->print_operation = gtk_print_operation_new ();
 	gtk_print_operation_set_allow_async (self->priv->print_operation, TRUE);
-	gtk_print_operation_set_custom_tab_label (self->priv->print_operation, _("Layout"));
+	gtk_print_operation_set_custom_tab_label (self->priv->print_operation, _("Images"));
 	gtk_print_operation_set_embed_page_setup (self->priv->print_operation, TRUE);
 	gtk_print_operation_set_show_progress (self->priv->print_operation, TRUE);
 
@@ -1584,16 +1863,53 @@ load_image_info_task_completed_cb (GthTask  *task,
 				   GError   *error,
 				   gpointer  user_data)
 {
-	GthImagePrintJob        *self = user_data;
-	GtkPrintOperationResult  result;
-	char                    *filename;
-	GtkPrintSettings        *settings;
+	GthImagePrintJob         *self = user_data;
+	int                       n_loaded_images;
+	GthImageInfo            **loaded_images;
+	int                       i, j;
+	GtkPrintOperationResult   result;
+	char                     *filename;
+	GtkPrintSettings         *settings;
 
 	if (error != NULL) {
-		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (self->priv->browser), _("Could not print"), &error);
+		g_object_unref (self);
 		return;
 	}
 
+	n_loaded_images = 0;
+	for (i = 0; i < self->priv->n_images; i++) {
+		GthImageInfo *image_info = self->priv->images[i];
+
+		if (image_info->thumbnail == NULL) {
+			gth_image_info_unref (self->priv->images[i]);
+			self->priv->images[i] = NULL;
+		}
+		else
+			n_loaded_images += 1;
+	}
+
+	if (n_loaded_images == 0) {
+		_gtk_error_dialog_show (GTK_WINDOW (self->priv->browser),
+					_("Could not print"),
+					"%s",
+					_("No suitable loader available for this file type"));
+		g_object_unref (self);
+		return;
+	}
+
+	loaded_images = g_new (GthImageInfo *, n_loaded_images + 1);
+	for (i = 0, j = 0; i < self->priv->n_images; i++) {
+		if (self->priv->images[i] != NULL) {
+			loaded_images[j] = self->priv->images[i];
+			j += 1;
+		}
+	}
+	loaded_images[j] = NULL;
+
+	g_free (self->priv->images);
+	self->priv->images = loaded_images;
+	self->priv->n_images = n_loaded_images;
+
 	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, "gthumb", "print_settings", NULL);
 	settings = gtk_print_settings_new_from_file (filename, NULL);
 	if (settings != NULL)
