@@ -238,22 +238,10 @@ for_each_child_done (ForEachChildData *fec)
 static void for_each_child_start_current (ForEachChildData *fec);
 
 
-static gboolean
-for_each_child_start_cb (gpointer user_data)
-{
-	ForEachChildData *fec = user_data;
-
-	g_source_remove (fec->source_id);
-	for_each_child_start_current (fec);
-
-	return FALSE;
-}
-
-
 static void
 for_each_child_start (ForEachChildData *fec)
 {
-	fec->source_id = g_idle_add (for_each_child_start_cb, fec);
+	for_each_child_start_current (fec);
 }
 
 
@@ -374,13 +362,16 @@ for_each_child_compute_child (ForEachChildData *fec,
 
 
 static void
-for_each_child_metadata_ready_func (GList    *files,
-				    GError   *error,
-				    gpointer  user_data)
+for_each_child_metadata_ready_func (GObject      *source_object,
+                		    GAsyncResult *result,
+                		    gpointer      user_data)
 {
 	ForEachChildData *fec = user_data;
+	GList            *files;
+	GError           *error = NULL;
 
-	if (error == NULL) {
+	files = _g_query_metadata_finish (result, &error);
+	if (files != NULL) {
 		GthFileData *child_data = files->data;
 		for_each_child_compute_child (fec, child_data->file, child_data->info);
 	}
@@ -1615,7 +1606,7 @@ copy_data__copy_current_file_ready_cb (GthOverwriteResponse  response,
 		GthFileData *source = (GthFileData *) copy_data->current->data;
 		copy_data->copied_files = g_list_prepend (copy_data->copied_files, g_file_dup (source->file));
 	}
-	else if (! g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+	else if ((response == GTH_OVERWRITE_RESPONSE_ALWAYS_NO) || ! g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
 		copy_data->done_callback (error, copy_data->user_data);
 		copy_data_free (copy_data);
 		return;
@@ -1645,6 +1636,12 @@ copy_data__copy_current_file (CopyData *copy_data)
 	}
 	destination = _g_file_get_destination (source->file, copy_data->source_base, copy_data->destination);
 
+	if (g_file_equal (source->file, destination)) {
+		g_object_unref (destination);
+		call_when_idle ((DataFunc) copy_data__copy_next_file, copy_data);
+		return;
+	}
+
 	flags = copy_data->flags;
 	if ((flags & G_FILE_COPY_ALL_METADATA) && (g_hash_table_lookup (copy_data->source_hash, source->file) == NULL))
 		flags = flags ^ G_FILE_COPY_ALL_METADATA;
@@ -1701,18 +1698,19 @@ copy_files__sources_info_ready_cb (GList    *files,
 
 
 void
-_g_copy_files_async (GList            *sources, /* GFile list */
-		     GFile            *destination,
-		     gboolean          move,
-		     GFileCopyFlags    flags,
-		     int               io_priority,
-		     GCancellable     *cancellable,
-		     ProgressCallback  progress_callback,
-		     gpointer          progress_callback_data,
-		     DialogCallback    dialog_callback,
-		     gpointer          dialog_callback_data,
-		     ReadyFunc         done_callback,
-		     gpointer          user_data)
+_g_copy_files_async (GList                *sources, /* GFile list */
+		     GFile                *destination,
+		     gboolean              move,
+		     GFileCopyFlags        flags,
+		     GthOverwriteResponse  default_response,
+		     int                   io_priority,
+		     GCancellable         *cancellable,
+		     ProgressCallback      progress_callback,
+		     gpointer              progress_callback_data,
+		     DialogCallback        dialog_callback,
+		     gpointer              dialog_callback_data,
+		     ReadyFunc             done_callback,
+		     gpointer              user_data)
 {
 	CopyData *copy_data;
 	GList    *scan;
@@ -1729,7 +1727,7 @@ _g_copy_files_async (GList            *sources, /* GFile list */
 	copy_data->dialog_callback_data = dialog_callback_data;
 	copy_data->done_callback = done_callback;
 	copy_data->user_data = user_data;
-	copy_data->default_response = GTH_OVERWRITE_RESPONSE_UNSPECIFIED;
+	copy_data->default_response = default_response;
 
 	copy_data->source_hash = g_hash_table_new_full ((GHashFunc) g_file_hash, (GEqualFunc) g_file_equal, (GDestroyNotify) g_object_unref, NULL);
 	for (scan = sources; scan; scan = scan->next)
@@ -2258,7 +2256,6 @@ _g_file_create_unique (GFile       *parent,
 	int                n;
 	GFileOutputStream *stream = NULL;
 
-	file = g_file_get_child_for_display_name (parent, display_name, &local_error);
 	n = 0;
 	do {
 		char *new_display_name;
@@ -2284,6 +2281,8 @@ _g_file_create_unique (GFile       *parent,
 	while ((stream == NULL) && (local_error == NULL));
 
 	if (stream == NULL) {
+		if (error != NULL)
+			*error = local_error;
 		g_object_unref (file);
 		file = NULL;
 	}
