@@ -377,6 +377,8 @@ gth_file_source_exec_next_in_queue (GthFileSource *file_source)
 	if (file_source->priv->queue == NULL)
 		return;
 
+	g_cancellable_reset (file_source->priv->cancellable);
+
 	head = file_source->priv->queue;
 	file_source->priv->queue = g_list_remove_link (file_source->priv->queue, head);
 
@@ -631,7 +633,8 @@ base_rename (GthFileSource *file_source,
 
 
 static gboolean
-base_can_cut (GthFileSource *file_source)
+base_can_cut (GthFileSource *file_source,
+	      GFile         *file)
 {
 	return FALSE;
 }
@@ -889,6 +892,7 @@ gth_file_source_write_metadata (GthFileSource *file_source,
 		gth_file_source_queue_write_metadata (file_source, file_data, attributes, callback, data);
 		return;
 	}
+	g_cancellable_reset (file_source->priv->cancellable);
 	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->write_metadata (file_source, file_data, attributes, callback, data);
 }
 
@@ -904,6 +908,7 @@ gth_file_source_read_metadata (GthFileSource *file_source,
 		gth_file_source_queue_read_metadata (file_source, file_data, attributes, callback, data);
 		return;
 	}
+	g_cancellable_reset (file_source->priv->cancellable);
 	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->read_metadata (file_source, file_data, attributes, callback, data);
 }
 
@@ -976,6 +981,8 @@ gth_file_source_list (GthFileSource *file_source,
 		return;
 	}
 
+	g_cancellable_reset (file_source->priv->cancellable);
+
 	data = g_new0 (ListOpData, 1);
 	data->file_source = g_object_ref (file_source);
 	data->ready_func = func;
@@ -1006,6 +1013,7 @@ gth_file_source_for_each_child (GthFileSource        *file_source,
 		gth_file_source_queue_for_each_child (file_source, parent, recursive, attributes, dir_func, child_func, ready_func, data);
 		return;
 	}
+	g_cancellable_reset (file_source->priv->cancellable);
 	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->for_each_child (file_source, parent, recursive, attributes, dir_func, child_func, ready_func, data);
 }
 
@@ -1072,6 +1080,8 @@ gth_file_source_read_attributes (GthFileSource  *file_source,
 		return;
 	}
 
+	g_cancellable_reset (file_source->priv->cancellable);
+
 	data = g_new0 (ReadAttributesOpData, 1);
 	data->file_source = g_object_ref (file_source);
 	data->ready_func = func;
@@ -1093,14 +1103,15 @@ void
 gth_file_source_rename (GthFileSource  *file_source,
 			GFile          *file,
 			GFile          *new_file,
-			ReadyCallback   callback,
+			ReadyCallback   ready_callback,
 			gpointer        data)
 {
 	if (gth_file_source_is_active (file_source)) {
-		gth_file_source_queue_rename (file_source, file, new_file, callback, data);
+		gth_file_source_queue_rename (file_source, file, new_file, ready_callback, data);
 		return;
 	}
-	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->rename (file_source, file, new_file, callback, data);
+	g_cancellable_reset (file_source->priv->cancellable);
+	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->rename (file_source, file, new_file, ready_callback, data);
 }
 
 
@@ -1118,14 +1129,16 @@ gth_file_source_copy (GthFileSource    *file_source,
 		gth_file_source_queue_copy (file_source, destination, file_list, move, progress_callback, dialog_callback, ready_callback, data);
 		return;
 	}
+	g_cancellable_reset (file_source->priv->cancellable);
 	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->copy (file_source, destination, file_list, move, progress_callback, dialog_callback, ready_callback, data);
 }
 
 
 gboolean
-gth_file_source_can_cut (GthFileSource *file_source)
+gth_file_source_can_cut (GthFileSource *file_source,
+			 GFile         *file)
 {
-	return GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->can_cut (file_source);
+	return GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->can_cut (file_source, file);
 }
 
 
@@ -1165,6 +1178,7 @@ gth_file_source_reorder (GthFileSource *file_source,
 		gth_file_source_queue_reorder (file_source, destination, visible_files, files_to_move, dest_pos, callback, data);
 		return;
 	}
+	g_cancellable_reset (file_source->priv->cancellable);
 	GTH_FILE_SOURCE_GET_CLASS (G_OBJECT (file_source))->reorder (file_source, destination, visible_files, files_to_move, dest_pos, callback, data);
 }
 
